"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processFile = void 0;
const prettier_1 = __importDefault(require("prettier"));
const graphql_1 = require("graphql");
const traverse_1 = require("../fragments/traverse");
const closure_1 = require("../fragments/closure");
const emit_1 = require("../fragments/emit");
const stringify_1 = require("../fragments/stringify");
const re = () => /(function\s+(\w+)\s*\((?:.|\r\n|\n)+?)useFragment(?:<((?:.|\r\n|\n)+?)>)?\((?:\s|\r\n|\n)*?`((?:.|\r\n|\n)+?)`(?:\s|\r\n|\n)*?\);/gm;
function processFile(introspection, fileData, gqlFilename) {
    var _a;
    const regexp = re();
    let defs = [];
    let hasRun = false;
    const replaced = fileData.replace(regexp, (substr, pre, fn, type, literal) => {
        const src = "fragment Fragment " +
            literal
                .replace(/\.\.\.\$\{([^}]+?)\}/gm, (val, match) => `...json_${Buffer.from(JSON.stringify({ match }), "utf-8").toString("hex")}`)
                .replace(/\$\{([^}]+?)\}/gm, (val, match) => `...hex_${Buffer.from(match, "utf-8").toString("hex")}`)
                .trim();
        const node = (0, graphql_1.parse)(src);
        const [traversed] = (0, traverse_1.traverseTree)(node);
        const closure = (0, closure_1.parseClosures)(introspection, traversed);
        const typeStr = (0, emit_1.emitStringFromClosure)(closure);
        const typeName = `${fn}Fragment`;
        const def = stringify_1.Stringify.typeDef(typeName, typeStr);
        defs.push(`/*\nAuto-generated from useFragment in ${fn}.\n*/`);
        defs.push(`export ${def}`);
        const typeImport = `import("./${gqlFilename}").${fn}Fragment`;
        const printedQuery = "\n" +
            (0, graphql_1.print)(node)
                .replace(/\.\.\.hex_(\w+)/g, (val, match) => `\$\{${Buffer.from(match, "hex").toString("utf-8")}\}`)
                .replace(/\.\.\.json_(\w+)/g, (val, match) => {
                const str = Buffer.from(match, "hex").toString("utf-8");
                return `...\$\{${JSON.parse(str).match}\}`;
            })
                .replace(/^fragment \w+/, "")
                .trim();
        hasRun = true;
        return `${pre}useFragment<${typeImport}>(\`${stringify_1.Stringify.indent(printedQuery, 4)}\n  \`);`;
    });
    if (!hasRun) {
        return {
            hasRun: false,
            definitions: "",
            parsed: null,
        };
    }
    const prettierConfigFile = prettier_1.default.resolveConfigFile.sync(process.cwd());
    const resolvedPrettierConfig = (_a = (prettierConfigFile != null
        ? prettier_1.default.resolveConfig.sync(prettierConfigFile)
        : null)) !== null && _a !== void 0 ? _a : {};
    let defFile = defs.join("\n");
    defFile = prettier_1.default.format(defFile, Object.assign(Object.assign({}, resolvedPrettierConfig), { parser: "typescript" }));
    let parsedFile = prettier_1.default.format(replaced, Object.assign(Object.assign({}, resolvedPrettierConfig), { parser: "typescript" }));
    return {
        hasRun: true,
        definitions: defFile,
        parsed: parsedFile,
    };
}
exports.processFile = processFile;
//# sourceMappingURL=process.js.map