"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupFragmentWatcher = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const chokidar_1 = __importDefault(require("chokidar"));
const glob_1 = __importDefault(require("glob"));
const process_1 = require("./process");
const schema_1 = require("../schema/schema");
function setupFragmentWatcher(globStr, cwd, introspection) {
    (0, schema_1.prepareSchemaForIntrospection)(introspection); // warm cache
    const watcher = chokidar_1.default.watch(globStr, {
        ignored: /\.gql(\.d)?\.tsx?$/,
        cwd,
        persistent: true,
    });
    const filesWithFragments = new Set();
    const emittedDefinitionFiles = new Set();
    function run(fileBase) {
        const file = (0, path_1.join)(cwd, fileBase);
        console.time(file);
        try {
            const ext = (0, path_1.extname)(file);
            const gqlFilename = (0, path_1.basename)(file, ext) + ".gql";
            const gqlFullname = (0, path_1.join)((0, path_1.dirname)(file), gqlFilename + ".d.ts");
            const input = (0, fs_1.readFileSync)(file, "utf-8");
            let result = (0, process_1.processFile)(introspection, input, gqlFilename);
            if (result.hasRun) {
                filesWithFragments.add(file);
                emittedDefinitionFiles.add(gqlFullname);
            }
            if (result.parsed != null && result.parsed !== input) {
                (0, fs_1.writeFileSync)(file, result.parsed);
            }
            if (result.definitions.trim().length > 0) {
                (0, fs_1.writeFileSync)(gqlFullname, result.definitions);
            }
            else {
                if ((0, fs_1.existsSync)(gqlFullname)) {
                    (0, fs_1.unlinkSync)(gqlFullname);
                }
            }
        }
        finally {
            console.timeEnd(file);
        }
    }
    // const existing = glob("**/*.gql.{ts")
    watcher
        .on("add", (path) => {
        run(path);
    })
        .on("ready", () => {
        const result = glob_1.default
            .sync("**/*.gql.*", {
            cwd,
        })
            .map((file) => (0, path_1.join)(cwd, file));
        for (const file of result) {
            if (!emittedDefinitionFiles.has(file)) {
                console.log("removing", file);
                (0, fs_1.unlinkSync)(file);
            }
        }
    })
        .on("change", (path) => {
        run(path);
    });
}
exports.setupFragmentWatcher = setupFragmentWatcher;
//# sourceMappingURL=watch.js.map