"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseClosures = void 0;
const schema_1 = require("../schema/schema");
const stringify_1 = require("./stringify");
function parseClosures(introspection, block) {
    const { getPossibleTypes, getTypeFromSchema } = (0, schema_1.prepareSchemaForIntrospection)(introspection);
    function parseClosure(block) {
        var _a;
        const type = getTypeFromSchema(block.currentType, block.currentPath);
        const closure = {
            possibleTypes: (type === null || type === void 0 ? void 0 : type.kind) === "referenceArray" || (type === null || type === void 0 ? void 0 : type.kind) === "reference"
                ? (_a = getPossibleTypes(type.type)) !== null && _a !== void 0 ? _a : []
                : [],
            hasTypename: false,
            branchedFields: new Map(),
            commonFields: new Map(),
            unionWith: block.spreadTypes,
        };
        function traverseBlock(mode, block, fieldMap) {
            var _a, _b, _c;
            for (const field of block.fields) {
                if (field.name === "__typename") {
                    if (mode === "anonymous") {
                        closure.hasTypename = true;
                        continue;
                    }
                    const possibilities = getPossibleTypes(field.currentType);
                    fieldMap.set(field.name, {
                        kind: "scalar",
                        name: "__typename",
                        type: stringify_1.Stringify.stringUnionType(possibilities !== null && possibilities !== void 0 ? possibilities : [], false),
                    });
                    continue;
                }
                if (fieldMap.has(field.name)) {
                    continue;
                }
                const foundType = getTypeFromSchema(field.currentType, field.currentPath);
                if (field.type === "object") {
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "reference") {
                        fieldMap.set(field.name, {
                            kind: "object",
                            closure: parseClosure(field.block),
                            name: field.name,
                            nullable: (_a = foundType === null || foundType === void 0 ? void 0 : foundType.nullable) !== null && _a !== void 0 ? _a : false,
                            array: false,
                        });
                    }
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "referenceArray") {
                        fieldMap.set(field.name, {
                            kind: "object",
                            closure: parseClosure(field.block),
                            name: field.name,
                            nullable: (_b = foundType === null || foundType === void 0 ? void 0 : foundType.nullable) !== null && _b !== void 0 ? _b : false,
                            array: true,
                            nullableArray: (_c = foundType === null || foundType === void 0 ? void 0 : foundType.nullableArray) !== null && _c !== void 0 ? _c : false,
                        });
                    }
                }
                if (field.type === "scalar") {
                    let type = "unknown";
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "scalar") {
                        type = stringify_1.Stringify.type(foundType.type, foundType.nullable);
                    }
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "scalarArray") {
                        type = stringify_1.Stringify.arrayType(foundType.type, foundType.nullable, foundType.nullableArray);
                    }
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "enum") {
                        type = stringify_1.Stringify.stringUnionType(foundType.options, foundType.nullable);
                    }
                    if ((foundType === null || foundType === void 0 ? void 0 : foundType.kind) === "enumArray") {
                        type = stringify_1.Stringify.arrayStringUnionType(foundType.options, foundType.nullable, foundType.nullableArray);
                    }
                    fieldMap.set(field.name, {
                        kind: "scalar",
                        name: field.name,
                        type,
                    });
                }
            }
            for (const anonymousFragment of block.anonymousFragments) {
                traverseBlock("anonymous", anonymousFragment, closure.commonFields);
            }
            for (const namedFragment of block.namedFragments) {
                const possibleTypes = getPossibleTypes(namedFragment.currentType);
                if (possibleTypes == null)
                    continue;
                for (const possible of possibleTypes) {
                    let map = closure.branchedFields.get(possible);
                    if (map == null) {
                        map = new Map();
                        closure.branchedFields.set(possible, map);
                    }
                    traverseBlock("named", namedFragment, map);
                }
            }
        }
        traverseBlock("anonymous", block, closure.commonFields);
        return closure;
    }
    return parseClosure(block);
}
exports.parseClosures = parseClosures;
//# sourceMappingURL=closure.js.map