"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitStringFromClosure = void 0;
const stringify_1 = require("./stringify");
function emitStringFromClosure(closure) {
    // console.dir(closure, { depth: 10 });
    function collectFields(fields) {
        const outFields = [];
        for (const [name, field] of fields) {
            if (field.kind === "scalar") {
                if (name === "__typename") {
                    outFields.unshift(stringify_1.Stringify.field(name, field.type));
                }
                else {
                    outFields.push(stringify_1.Stringify.field(name, field.type));
                }
            }
            if (field.kind === "object") {
                const type = recurseClosure(field.closure);
                if (field.array) {
                    outFields.push(stringify_1.Stringify.field(name, `Array<${stringify_1.Stringify.unionType([type], field.nullable)}>` +
                        (field.nullableArray ? " | null" : "")));
                }
                else {
                    outFields.push(stringify_1.Stringify.field(name, stringify_1.Stringify.unionType([type], field.nullable)));
                }
            }
        }
        return outFields;
    }
    function recurseClosure(closure) {
        const commonFields = collectFields(closure.commonFields);
        const branches = [];
        const possibleTypes = new Set(closure.possibleTypes);
        for (const [name, fields] of closure.branchedFields) {
            const branchFields = collectFields(fields);
            if (closure.hasTypename && !fields.has("__typename")) {
                branchFields.unshift(stringify_1.Stringify.field("__typename", `"${name}"`));
            }
            const branchType = stringify_1.Stringify.objectType(...branchFields);
            branches.push(branchType);
            possibleTypes.delete(name);
        }
        if (possibleTypes.size > 0) {
            if (closure.hasTypename) {
                for (const possible of possibleTypes) {
                    branches.unshift(stringify_1.Stringify.objectType(stringify_1.Stringify.field("__typename", `"${possible}"`)));
                }
            }
            else if (closure.branchedFields.size > 0) {
                branches.unshift("{}");
            }
        }
        let base = stringify_1.Stringify.objectType(...commonFields);
        if (branches.length > 0) {
            base = stringify_1.Stringify.and(base, stringify_1.Stringify.or(...branches));
        }
        if (closure.unionWith.length > 0) {
            base = stringify_1.Stringify.and(base, ...closure.unionWith.map((el) => `${el}Fragment`));
        }
        return base;
    }
    return recurseClosure(closure);
    // return Stringify.typeDef("Type", str); // format(, { parser: "typescript" });
}
exports.emitStringFromClosure = emitStringFromClosure;
//# sourceMappingURL=emit.js.map