"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stringify = void 0;
var Stringify;
(function (Stringify) {
    function indent(str, levels = 2) {
        return str.replace(/\n/g, "\n".padEnd(levels + 1, " "));
    }
    Stringify.indent = indent;
    function typeDef(name, def) {
        return `type ${name} = ${indent(def)};`;
    }
    Stringify.typeDef = typeDef;
    function and(...spreads) {
        return spreads
            .filter((el) => el.length > 0 && el !== "{}")
            .map((el) => `(\n  ${indent(el)}\n)`)
            .join(" & ");
    }
    Stringify.and = and;
    function or(...spreads) {
        return spreads
            .filter((el) => el.length > 0)
            .map((el) => `(\n  ${indent(el)}\n)`)
            .join(" | ");
    }
    Stringify.or = or;
    function objectType(...fields) {
        if (fields.length === 0) {
            return "{}";
        }
        return `{\n  ${fields.map((el) => indent(el)).join(";\n  ")}\n}`;
    }
    Stringify.objectType = objectType;
    function field(name, type, operator = ":") {
        return `${name}${operator} ${type}`;
    }
    Stringify.field = field;
    function type(type, nullable = false) {
        return `${type.length > 0 ? type : "unknown"}${nullable ? " | null" : ""}`;
    }
    Stringify.type = type;
    function arrayType(type, nullable = false, nullableArray = false) {
        const baseType = `${type}${nullable ? " | null" : ""}`;
        return `Array<${baseType}>${nullableArray ? " | null" : ""}`;
    }
    Stringify.arrayType = arrayType;
    function unionType(choices, nullable = false) {
        return type(choices.join(" | "), nullable);
    }
    Stringify.unionType = unionType;
    function stringUnionType(choices, nullable = false) {
        return unionType(choices.map((el) => `"${el}"`), nullable);
    }
    Stringify.stringUnionType = stringUnionType;
    function arrayScalarUnionType(choices, nullable = false, nullableArray = false) {
        return arrayType(choices.join(" | "), nullable, nullableArray);
    }
    Stringify.arrayScalarUnionType = arrayScalarUnionType;
    function arrayStringUnionType(choices, nullable = false, nullableArray = false) {
        return arrayScalarUnionType(choices.map((el) => `"${el}"`), nullable, nullableArray);
    }
    Stringify.arrayStringUnionType = arrayStringUnionType;
    function typePath(currentType, currentPath) {
        const currentPathStack = currentPath.map((el) => `["${el}"]`).join("");
        return `Types.GQL${currentType}${currentPathStack}`;
    }
    Stringify.typePath = typePath;
})(Stringify = exports.Stringify || (exports.Stringify = {}));
/*
console.log(
  Stringify.typeDef(
    "name",
    Stringify.or(
      "test",
      "test",
      "test",
      Stringify.and(
        "test",
        "test2",
        Stringify.field(
          "test",
          Stringify.objectType(
            Stringify.field("test", "type", "?:"),
            Stringify.field(
              "test",
              Stringify.objectType(
                Stringify.field("test", "type", "?:"),
                Stringify.field(
                  "test",
                  Stringify.objectType(
                    Stringify.field("test", "type", "?:"),
                    Stringify.field(
                      "test",
                      Stringify.objectType("test"),
                      Stringify.field("test", "type")
                    )
                  )
                )
              )
            )
          )
        )
      )
    )
  )
);
*/
//# sourceMappingURL=stringify.js.map