"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareSchemaForIntrospection = void 0;
const parse_1 = require("./parse");
function _prepareSchemaForIntrospection(data) {
    const parsed = (0, parse_1.parseSchema)(data);
    function getTypeFromSchema(type, path) {
        var _a, _b;
        let fields = null;
        let foundType = (_b = (_a = parsed.objects.find((el) => el.name === type)) !== null && _a !== void 0 ? _a : parsed.interfaces.find((el) => el.name === type)) !== null && _b !== void 0 ? _b : null;
        if (foundType != null) {
            fields = foundType.fields;
        }
        if (fields == null) {
            return null;
        }
        const nextPath = [...path];
        const fieldSelect = nextPath.shift();
        if (fieldSelect != null) {
            const field = fields.find((el) => el.name === fieldSelect);
            if (field == null) {
                return null;
            }
            if (nextPath.length === 0) {
                // need to select enums or scalar fields here...
                if (field.builtIn) {
                    if (field.array) {
                        return {
                            kind: "scalarArray",
                            type: field.type,
                            nullable: field.nullable,
                            nullableArray: field.nullableArray,
                        };
                    }
                    return {
                        kind: "scalar",
                        type: field.type,
                        nullable: field.nullable,
                    };
                }
                if (field.isEnum) {
                    const enumType = parsed.enums.find((el) => el.name === field.type);
                    if (field.array) {
                        return {
                            kind: "enumArray",
                            options: enumType.values.map((el) => el.value),
                            nullable: field.nullable,
                            nullableArray: field.nullableArray,
                        };
                    }
                    return {
                        kind: "enum",
                        options: enumType.values.map((el) => el.value),
                        nullable: field.nullable,
                    };
                }
                if (field.array) {
                    return {
                        kind: "referenceArray",
                        type: field.type,
                        nullable: field.nullable,
                        nullableArray: field.nullableArray,
                    };
                }
                return {
                    kind: "reference",
                    type: field.type,
                    nullable: field.nullable,
                };
            }
            return getTypeFromSchema(field.type, nextPath);
        }
        if (foundType != null && path.length === 0) {
            return {
                kind: "reference",
                type: foundType.name,
                nullable: false,
            };
        }
        return null;
    }
    function getPossibleTypes(type) {
        const foundObjectType = parsed.objects.find((el) => el.name === type);
        if (foundObjectType != null) {
            return foundObjectType.possibleTypes;
        }
        const foundInterfaceType = parsed.interfaces.find((el) => el.name === type);
        if (foundInterfaceType != null) {
            return foundInterfaceType.possibleTypes;
        }
        const foundUnionType = parsed.unions.find((el) => el.name === type);
        if (foundUnionType != null) {
            return foundUnionType.unionOf;
        }
        return null;
    }
    /*
    console.log(getTypeFromSchema("FieldA", ["a"]));
    console.log(getTypeFromSchema("FieldA", ["b"]));
    console.log(getTypeFromSchema("FieldA", ["b", "num"]));
    console.log(getTypeFromSchema("FieldA", ["c"]));
    console.log(getTypeFromSchema("FieldA", ["d"]));
  
    console.log(getPossibleTypes("FieldA"));
    console.log(getPossibleTypes("FieldB"));
    console.log(getPossibleTypes("FieldC"));
    console.log(getPossibleTypes("FieldD"));
    console.log(getPossibleTypes("Base"));
    console.log(getPossibleTypes("Test"));
    */
    return {
        getPossibleTypes,
        getTypeFromSchema,
    };
}
let cache = null;
function prepareSchemaForIntrospection(query) {
    if (cache == null || cache.query !== query) {
        cache = {
            result: _prepareSchemaForIntrospection(query),
            query,
        };
    }
    return cache.result;
}
exports.prepareSchemaForIntrospection = prepareSchemaForIntrospection;
//# sourceMappingURL=schema.js.map