import prettier from "prettier";
import { parse, print } from "graphql";
import { traverseTree } from "../fragments/traverse";
import { parseClosures } from "../fragments/closure";
import { emitStringFromClosure } from "../fragments/emit";
import { Stringify } from "../fragments/stringify";
const re = () => /(function\s+(\w+)\s*\((?:.|\r\n|\n)+?)useFragment(?:<((?:.|\r\n|\n)+?)>)?\((?:\s|\r\n|\n)*?`((?:.|\r\n|\n)+?)`(?:\s|\r\n|\n)*?\);/gm;
export function processFile(introspection, fileData, gqlFilename) {
    var _a;
    const regexp = re();
    let defs = [];
    let hasRun = false;
    const replaced = fileData.replace(regexp, (substr, pre, fn, type, literal) => {
        const src = "fragment Fragment " +
            literal
                .replace(/\.\.\.\$\{([^}]+?)\}/gm, (val, match) => `...json_${Buffer.from(JSON.stringify({ match }), "utf-8").toString("hex")}`)
                .replace(/\$\{([^}]+?)\}/gm, (val, match) => `...hex_${Buffer.from(match, "utf-8").toString("hex")}`)
                .trim();
        const node = parse(src);
        const [traversed] = traverseTree(node);
        const closure = parseClosures(introspection, traversed);
        const typeStr = emitStringFromClosure(closure);
        const typeName = `${fn}Fragment`;
        const def = Stringify.typeDef(typeName, typeStr);
        defs.push(`/*\nAuto-generated from useFragment in ${fn}.\n*/`);
        defs.push(`export ${def}`);
        const typeImport = `import("./${gqlFilename}").${fn}Fragment`;
        const printedQuery = "\n" +
            print(node)
                .replace(/\.\.\.hex_(\w+)/g, (val, match) => `\$\{${Buffer.from(match, "hex").toString("utf-8")}\}`)
                .replace(/\.\.\.json_(\w+)/g, (val, match) => {
                const str = Buffer.from(match, "hex").toString("utf-8");
                return `...\$\{${JSON.parse(str).match}\}`;
            })
                .replace(/^fragment \w+/, "")
                .trim();
        hasRun = true;
        return `${pre}useFragment<${typeImport}>(\`${Stringify.indent(printedQuery, 4)}\n  \`);`;
    });
    if (!hasRun) {
        return {
            hasRun: false,
            definitions: "",
            parsed: null,
        };
    }
    const prettierConfigFile = prettier.resolveConfigFile.sync(process.cwd());
    const resolvedPrettierConfig = (_a = (prettierConfigFile != null
        ? prettier.resolveConfig.sync(prettierConfigFile)
        : null)) !== null && _a !== void 0 ? _a : {};
    let defFile = defs.join("\n");
    defFile = prettier.format(defFile, Object.assign(Object.assign({}, resolvedPrettierConfig), { parser: "typescript" }));
    let parsedFile = prettier.format(replaced, Object.assign(Object.assign({}, resolvedPrettierConfig), { parser: "typescript" }));
    return {
        hasRun: true,
        definitions: defFile,
        parsed: parsedFile,
    };
}
//# sourceMappingURL=process.js.map