import { readFileSync, writeFileSync, existsSync, unlinkSync } from "fs";
import { join, dirname, basename, extname } from "path";
import chokidar from "chokidar";
import glob from "glob";
import { processFile } from "./process";
import { prepareSchemaForIntrospection } from "../schema/schema";
export function setupFragmentWatcher(globStr, cwd, introspection) {
    prepareSchemaForIntrospection(introspection); // warm cache
    const watcher = chokidar.watch(globStr, {
        ignored: /\.gql(\.d)?\.tsx?$/,
        cwd,
        persistent: true,
    });
    const filesWithFragments = new Set();
    const emittedDefinitionFiles = new Set();
    function run(fileBase) {
        const file = join(cwd, fileBase);
        console.time(file);
        try {
            const ext = extname(file);
            const gqlFilename = basename(file, ext) + ".gql";
            const gqlFullname = join(dirname(file), gqlFilename + ".d.ts");
            const input = readFileSync(file, "utf-8");
            let result = processFile(introspection, input, gqlFilename);
            if (result.hasRun) {
                filesWithFragments.add(file);
                emittedDefinitionFiles.add(gqlFullname);
            }
            if (result.parsed != null && result.parsed !== input) {
                writeFileSync(file, result.parsed);
            }
            if (result.definitions.trim().length > 0) {
                writeFileSync(gqlFullname, result.definitions);
            }
            else {
                if (existsSync(gqlFullname)) {
                    unlinkSync(gqlFullname);
                }
            }
        }
        finally {
            console.timeEnd(file);
        }
    }
    // const existing = glob("**/*.gql.{ts")
    watcher
        .on("add", (path) => {
        run(path);
    })
        .on("ready", () => {
        const result = glob
            .sync("**/*.gql.*", {
            cwd,
        })
            .map((file) => join(cwd, file));
        for (const file of result) {
            if (!emittedDefinitionFiles.has(file)) {
                console.log("removing", file);
                unlinkSync(file);
            }
        }
    })
        .on("change", (path) => {
        run(path);
    });
}
//# sourceMappingURL=watch.js.map