function traverseHasTypename(nodes) {
    for (const node of nodes) {
        if (node.kind !== "Field")
            continue;
        if (node.name.value === "__typename") {
            return true;
        }
    }
    return false;
}
function traverseAnonymousFragments(nodes, currentType, currentPath) {
    const blocks = [];
    for (const node of nodes) {
        if (node.kind !== "InlineFragment")
            continue;
        if (node.typeCondition != null)
            continue;
        traverseFragmentSpreads(node.selectionSet.selections, currentType, currentPath);
        blocks.push({
            currentType,
            currentPath,
            hasTypename: traverseHasTypename(node.selectionSet.selections),
            anonymousFragments: traverseAnonymousFragments(node.selectionSet.selections, currentType, currentPath),
            fields: traverseFields(node.selectionSet.selections, currentType, currentPath),
            namedFragments: traverseNamedFragments(node.selectionSet.selections, currentType, currentPath),
            spreadTypes: traverseFragmentSpreads(node.selectionSet.selections, currentType, currentPath),
        });
    }
    return blocks;
}
function traverseNamedFragments(nodes, currentType, currentPath) {
    const blocks = [];
    for (const node of nodes) {
        if (node.kind !== "InlineFragment")
            continue;
        if (node.typeCondition == null)
            continue;
        traverseFragmentSpreads(node.selectionSet.selections, currentType, currentPath);
        blocks.push({
            currentType: node.typeCondition.name.value,
            currentPath: [],
            hasTypename: traverseHasTypename(node.selectionSet.selections),
            anonymousFragments: traverseAnonymousFragments(node.selectionSet.selections, node.typeCondition.name.value, []),
            fields: traverseFields(node.selectionSet.selections, node.typeCondition.name.value, []),
            namedFragments: traverseNamedFragments(node.selectionSet.selections, node.typeCondition.name.value, []),
            spreadTypes: traverseFragmentSpreads(node.selectionSet.selections, currentType, currentPath),
        });
    }
    return blocks;
}
function traverseFields(nodes, currentType, currentPath) {
    const fields = [];
    for (const node of nodes) {
        if (node.kind !== "Field")
            continue;
        const newPath = [...currentPath, node.name.value];
        if (node.selectionSet != null) {
            fields.push({
                type: "object",
                name: node.name.value,
                block: {
                    currentType,
                    currentPath: newPath,
                    hasTypename: traverseHasTypename(node.selectionSet.selections),
                    anonymousFragments: traverseAnonymousFragments(node.selectionSet.selections, currentType, newPath),
                    fields: traverseFields(node.selectionSet.selections, currentType, newPath),
                    namedFragments: traverseNamedFragments(node.selectionSet.selections, currentType, newPath),
                    spreadTypes: traverseFragmentSpreads(node.selectionSet.selections, currentType, currentPath),
                },
                currentType,
                currentPath: newPath,
            });
        }
        else {
            fields.push({
                type: "scalar",
                name: node.name.value,
                currentType,
                currentPath: newPath,
            });
        }
    }
    return fields;
}
function traverseFragmentSpreads(nodes, currentType, currentPath) {
    const types = [];
    for (const node of nodes) {
        if (node.kind !== "FragmentSpread")
            continue;
        if (node.name.value.startsWith("json_")) {
            const rest = node.name.value.slice(5);
            const str = Buffer.from(rest, "hex").toString("utf-8");
            types.push(JSON.parse(str).match);
        }
    }
    return types;
}
export function traverseTree(node) {
    const blocks = [];
    for (const def of node.definitions) {
        if (def.kind !== "FragmentDefinition")
            continue;
        const currentType = def.typeCondition.name.value;
        const currentPath = [];
        blocks.push({
            currentType,
            currentPath,
            hasTypename: traverseHasTypename(def.selectionSet.selections),
            fields: traverseFields(def.selectionSet.selections, currentType, currentPath),
            anonymousFragments: traverseAnonymousFragments(def.selectionSet.selections, currentType, currentPath),
            namedFragments: traverseNamedFragments(def.selectionSet.selections, currentType, currentPath),
            spreadTypes: traverseFragmentSpreads(def.selectionSet.selections, currentType, currentPath),
        });
    }
    return blocks;
}
//# sourceMappingURL=traverse.js.map